namespace ODataTour { 
  using System;
  using System.Collections;
  using System.Data.Services;
  using System.Linq;
  using System.Reflection;
  using IdeaBlade.EntityModel;
  
  // <summary>
  // Implementation of System.Data.Services.IUpdatable to support OData updates.  This code is generated once, and may be modified as needed.
  // </summary>
  public partial class NorthwindIBEntities : IUpdatable {
  
    #region IUpdatable implementation
  
    /// <summary>
    /// Add an entity to a RelatedEntityList.
    /// </summary>
    /// <param name="targetResource"></param>
    /// <param name="propertyName"></param>
    /// <param name="resourceToBeAdded"></param>
    void IUpdatable.AddReferenceToCollection(object targetResource, string propertyName, object resourceToBeAdded) {
      IList collection = GetRelatedEntityList(targetResource, propertyName);
      collection.Add(resourceToBeAdded);
    }
  
    /// <summary>
    /// Cancel changes.
    /// </summary>
    void IUpdatable.ClearChanges() {
      RejectChanges();
    }
  
    /// <summary>
    /// Create an entity or complex object.
    /// </summary>
    /// <param name="containerName"></param>
    /// <param name="fullTypeName"></param>
    /// <returns></returns>
    object IUpdatable.CreateResource(string containerName, string fullTypeName) {
      Type t = Type.GetType(fullTypeName, true);
      object resource = Activator.CreateInstance(t);
      // Add the entity to the EntityManager.
      if ((resource as IComplexObject) == null) {
        AddEntity(resource);
      }
      return resource;
    }
  
    /// <summary>
    /// Delete the entity.
    /// </summary>
    /// <param name="targetResource"></param>
    void IUpdatable.DeleteResource(object targetResource) {
      var aspect = GetEntityAspect(targetResource);
      if (aspect == null) {
        throw new ApplicationException("Unexpected type for this resource");
      }
  
      aspect.Delete();
    }
  
    /// <summary>
    /// Gets the resource of the specified type identified by a query and type name.
    /// </summary>
    /// <param name="query"></param>
    /// <param name="fullTypeName"></param>
    /// <returns></returns>
    object IUpdatable.GetResource(IQueryable query, string fullTypeName) {
      var resources = query.GetEnumerator();
      resources.MoveNext();
      object resource = resources.Current;
  
      if (fullTypeName != null && resource.GetType().FullName != fullTypeName)
        throw new ApplicationException("Unexpected type for this resource");
  
      return resource;
    }
  
    /// <summary>
    /// Get a property value.
    /// </summary>
    /// <param name="targetResource"></param>
    /// <param name="propertyName"></param>
    /// <returns></returns>
    object IUpdatable.GetValue(object targetResource, string propertyName) {
      var prop = GetEntityProperty(targetResource, propertyName);
      return prop.GetValue(targetResource);
    }
  
    /// <summary>
    /// Remove an entity from a RelatedEntityList.
    /// </summary>
    /// <param name="targetResource"></param>
    /// <param name="propertyName"></param>
    /// <param name="resourceToBeRemoved"></param>
    void IUpdatable.RemoveReferenceFromCollection(object targetResource, string propertyName, object resourceToBeRemoved) {
      IList collection = GetRelatedEntityList(targetResource, propertyName);
      collection.Remove(resourceToBeRemoved);
    }
  
    /// <summary>
    /// Reset resource to original values.
    /// </summary>
    /// <param name="resource"></param>
    /// <returns></returns>
    object IUpdatable.ResetResource(object resource) {
      var aspect = GetEntityAspect(resource);
      if (aspect == null) {
        return aspect;
      }
  
      aspect.RejectChanges();
      return aspect.Entity;
    }
  
    /// <summary>
    /// Returns resource instance.
    /// </summary>
    /// <param name="resource"></param>
    /// <returns></returns>
    object IUpdatable.ResolveResource(object resource) {
      return resource;
    }
  
    /// <summary>
    /// Saves any outstanding changes.
    /// </summary>
    void IUpdatable.SaveChanges() {
      this.SaveChanges();
    }
  
    /// <summary>
    /// Set the value of a navigation property.
    /// </summary>
    /// <param name="targetResource"></param>
    /// <param name="propertyName"></param>
    /// <param name="propertyValue"></param>
    void IUpdatable.SetReference(object targetResource, string propertyName, object propertyValue) {
      ((IUpdatable)this).SetValue(targetResource, propertyName, propertyValue);
    }
  
    /// <summary>
    /// Sets the property value.
    /// </summary>
    /// <param name="targetResource"></param>
    /// <param name="propertyName"></param>
    /// <param name="propertyValue"></param>
    void IUpdatable.SetValue(object targetResource, string propertyName, object propertyValue) {
      // Property can be synthetic, so use both DF metadata and reflection.
      var prop = GetEntityProperty(targetResource, propertyName, false) as DataEntityProperty;
      PropertyInfo pi = (prop != null) ? prop.PropertyInfo : targetResource.GetType().GetProperty(propertyName);
      if (pi == null) {
         throw new ApplicationException("Property: " + propertyName + " not found");
      }
      // Disallow if readonly or an auto-incrementing key.
      if (IsPropertyReadOnly(pi) || (prop != null && (prop.IsAutoIncrementing || prop.IsReadOnly))) return;
      pi.SetValue(targetResource, propertyValue, null);
    }
    #endregion
  
    #region Helper functions
  
    private EntityProperty GetEntityProperty(object targetResource, string propertyName, bool throwIfNotFound = true) {
      var metadata = MetadataStore.GetEntityMetadata(targetResource.GetType());
      var prop = metadata.EntityProperties.FirstOrDefault(p => p.Name == propertyName);
      if (prop == null && throwIfNotFound) {
        throw new ApplicationException("Property: " + propertyName + " not found");
      }
      return prop;
    }
  
    private IList GetRelatedEntityList(object targetResource, string propertyName) {
      var prop = GetEntityProperty(targetResource, propertyName) as NavigationEntityProperty;
      if (prop == null || prop.IsCollectionReadOnly || prop.IsScalar) {
        throw new ApplicationException("Property: " + propertyName + " is not an updatable collection");
      }
      return (IList)prop.GetValue(targetResource);
    }
  
    private static EntityAspect GetEntityAspect(object targetResource) {
      if (targetResource is IComplexObject) return null;
      return EntityAspect.Wrap(targetResource);
    }
  
    private static bool IsPropertyReadOnly(PropertyInfo pi) {
      if (pi == null) return true;
      return pi.GetSetMethod(false) == null ? true : false;
    }
    #endregion
  }

}
